//=================================================================================================
//
//
// ntouch
//
// Touch Utility for Windows
//
//                                                                 (c) Florian Balmer 2001-2013
//                                                                  http://www.flos-freeware.ch
//
//
//=================================================================================================

#include <windows.h>
#include <commctrl.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <uxtheme.h>
#include <tchar.h>
#include "resource.h"


#define SC_ALWAYSONTOP 60001
//#define SC_INFO        60002


BOOL AddFiles(HWND hwnd,LPCTSTR lpszArgList,BOOL bCmdLine)
{

  BOOL bSuccess = TRUE;

  LVITEM lvi;

  LPTSTR *lpszArgs;
  int     iNumArgs;
  int     i,iStart;

  HANDLE hFind;
  WIN32_FIND_DATA fd;

  TCHAR szShortPath[MAX_PATH];
  TCHAR szLongPath[MAX_PATH];

  ZeroMemory(&lvi,sizeof(LVITEM));
  lvi.iItem  = ListView_GetItemCount(hwnd);
  lvi.mask   = LVIF_TEXT | LVIF_IMAGE;
  lvi.iImage = I_IMAGECALLBACK;

  lpszArgs = CommandLineToArgvW(lpszArgList,&iNumArgs);

  if ((bCmdLine && iNumArgs > 1) || (!bCmdLine && iNumArgs > 0))
  {

    iStart = (bCmdLine) ? 1 : 0;

    for (i = iStart; i < iNumArgs; i++)
    {

      if (PathFileExists(lpszArgs[i])) {

        if (PathIsRelative(lpszArgs[i])) {
          GetCurrentDirectory(MAX_PATH,szShortPath);
          PathAppend(szShortPath,lpszArgs[i]);
        }
        else
          StrCpyN(szShortPath,lpszArgs[i],MAX_PATH);

        if (!GetLongPathName(szShortPath,szLongPath,MAX_PATH))
          StrCpyN(szLongPath,szShortPath,MAX_PATH);

        if (PathFileExists(szLongPath) && !PathIsRoot(szLongPath)) {

          lvi.pszText = szLongPath;
          ListView_InsertItem(hwnd,&lvi);
          lvi.iItem++; }

      }

      else {

        hFind = FindFirstFile(lpszArgs[i],&fd);
        if (hFind != INVALID_HANDLE_VALUE)
        {

          PathRemoveFileSpec(lpszArgs[i]);

          do {

            if (StrCmp(fd.cFileName,TEXT(".")) &&
                StrCmp(fd.cFileName,TEXT(".."))) {

              if (PathIsRelative(lpszArgs[i])) {
                GetCurrentDirectory(MAX_PATH,szShortPath);
                PathAppend(szShortPath,lpszArgs[i]);
              }
              else
                StrCpyN(szShortPath,lpszArgs[i],MAX_PATH);

              PathAppend(szShortPath,fd.cFileName);

              if (!GetLongPathName(szShortPath,szLongPath,MAX_PATH))
                StrCpyN(szLongPath,szShortPath,MAX_PATH);

              if (PathFileExists(szLongPath) && !PathIsRoot(szLongPath)) {

                lvi.pszText = szLongPath;
                ListView_InsertItem(hwnd,&lvi);
                lvi.iItem++; }

            }

          } while (FindNextFile(hFind,&fd));

          FindClose(hFind);

        }
        else
          bSuccess = FALSE;
      }

    }

  }

  LocalFree(lpszArgs);

  if (!bCmdLine) {
    i = ListView_GetItemCount(hwnd) - 1;
    ListView_SetItemState(hwnd,i,LVIS_FOCUSED,LVIS_FOCUSED);
    ListView_EnsureVisible(hwnd,i,FALSE); }

  return(bSuccess);

}


BOOL SetFileTimeEx(LPTSTR lpFileName,LPFILETIME lpFileTime,BOOL bModifyCreationTime)
{

  BOOL   r_SetFileTime;

  HANDLE hFile = CreateFile(lpFileName,
                            GENERIC_WRITE,
                            0,
                            NULL,
                            OPEN_EXISTING,
                            FILE_FLAG_BACKUP_SEMANTICS,
                            NULL);

  if (hFile == INVALID_HANDLE_VALUE)
    return(FALSE);

  if (bModifyCreationTime)
    r_SetFileTime = SetFileTime(hFile,lpFileTime,lpFileTime,lpFileTime);
  else
    r_SetFileTime = SetFileTime(hFile,NULL,lpFileTime,lpFileTime);

  CloseHandle(hFile);

  return(r_SetFileTime);

}


BOOL CALLBACK MainDlgProc(HWND hwnd,UINT umsg,WPARAM wParam,LPARAM lParam)
{

  static int xDialog;
  static int yDialog;
  static int cxDialog;
  static int cyDialog;
  static int cxClient;
  static int cyClient;
  static int mmiPtMaxY;
  static int mmiPtMinX;

  static BOOL bAlwaysOnTop;
  static BOOL bUseCustomTime;
  static BOOL bModifyCreationTime;

  int nIdFocus;

  switch(umsg)
  {

    case WM_INITDIALOG:
      {
        RECT rc;
        RECT rcWorkArea;
        int cGrip;

        SHFILEINFO shfi;
        SYSTEMTIME st;
        LVCOLUMN lvc = { LVCF_FMT|LVCF_TEXT, LVCFMT_LEFT, 0, L"", -1, 0, 0, 0 };

        // Load ini settings
        TCHAR tchIniFile[MAX_PATH];
        GetModuleFileName(GetModuleHandle(NULL),tchIniFile,MAX_PATH);
        PathRenameExtension(tchIniFile,TEXT(".ini"));

        // Transform window rect from workarea to screen
        SystemParametersInfo(SPI_GETWORKAREA,0,&rcWorkArea,0);

        GetWindowRect(hwnd,&rc);
        xDialog = rcWorkArea.left + GetPrivateProfileInt(TEXT("Settings"),TEXT("WinPosX"),rc.left-rcWorkArea.left,tchIniFile);
        yDialog = rcWorkArea.top  + GetPrivateProfileInt(TEXT("Settings"),TEXT("WinPosY"),rc.top-rcWorkArea.top,tchIniFile);
        cxDialog = GetPrivateProfileInt(TEXT("Settings"),TEXT("WinSizeX"),0,tchIniFile);
        cyDialog = GetPrivateProfileInt(TEXT("Settings"),TEXT("WinSizeY"),0,tchIniFile);

        bAlwaysOnTop = GetPrivateProfileInt(TEXT("Settings"),TEXT("KeepOnTop"),0,tchIniFile) ? TRUE : FALSE;
        bUseCustomTime = GetPrivateProfileInt(TEXT("Settings"),TEXT("CustomTime"),0,tchIniFile) ? TRUE : FALSE;
        bModifyCreationTime = GetPrivateProfileInt(TEXT("Settings"),TEXT("ModifyCreationTime"),0,tchIniFile) ? TRUE : FALSE;

        // Setup dialog
        GetClientRect(hwnd,&rc);
        cxClient = rc.right - rc.left;
        cyClient = rc.bottom - rc.top;

        AdjustWindowRectEx(&rc,GetWindowLong(hwnd,GWL_STYLE)|WS_THICKFRAME,FALSE,0);
        mmiPtMinX = rc.right-rc.left;
        mmiPtMaxY = rc.bottom-rc.top;

        if (cxDialog < (rc.right-rc.left))
          cxDialog = rc.right-rc.left;
        if (cyDialog < (rc.bottom-rc.top))
          cyDialog = rc.bottom-rc.top;
        SetWindowPos(hwnd,NULL,xDialog,yDialog,cxDialog,cyDialog,SWP_NOZORDER);

        SetWindowLong(hwnd,GWL_STYLE,GetWindowLong(hwnd,GWL_STYLE)|WS_THICKFRAME);
        SetWindowPos(hwnd,NULL,0,0,0,0,SWP_NOZORDER|SWP_NOMOVE|SWP_NOSIZE|SWP_FRAMECHANGED);

        InsertMenu(GetSystemMenu(hwnd,FALSE),SC_MOVE,MF_BYCOMMAND|MF_STRING|MF_ENABLED,SC_ALWAYSONTOP,TEXT("&Keep on top"));
        InsertMenu(GetSystemMenu(hwnd,FALSE),SC_CLOSE,MF_BYCOMMAND|MF_STRING|MF_ENABLED,SC_SIZE,TEXT("&Size"));
        //InsertMenu(GetSystemMenu(hwnd,FALSE),SC_CLOSE,MF_BYCOMMAND|MF_SEPARATOR,0,NULL);
        //InsertMenu(GetSystemMenu(hwnd,FALSE),SC_CLOSE,MF_BYCOMMAND|MF_STRING|MF_ENABLED,SC_INFO,TEXT("&Info..."));
        InsertMenu(GetSystemMenu(hwnd,FALSE),SC_CLOSE,MF_BYCOMMAND|MF_SEPARATOR,0,NULL);

        SetWindowLong(GetDlgItem(hwnd,IDC_GRIPPER),GWL_STYLE,
          GetWindowLong(GetDlgItem(hwnd,IDC_GRIPPER),GWL_STYLE)|SBS_SIZEGRIP|WS_CLIPSIBLINGS);

        cGrip = GetSystemMetrics(SM_CXHTHUMB);
        SetWindowPos(GetDlgItem(hwnd,IDC_GRIPPER),NULL,cxClient-cGrip,
                     cyClient-cGrip,cGrip,cGrip,SWP_NOZORDER);

        if (bAlwaysOnTop)
          SetWindowPos(hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);

        // Setup controls
        GetLocalTime(&st);
        st.wMinute = 0;
        st.wSecond = 0;
        st.wMilliseconds = 0;
        DateTime_SetSystemtime(GetDlgItem(hwnd,IDC_DATE),GDT_VALID,&st);
        DateTime_SetSystemtime(GetDlgItem(hwnd,IDC_TIME),GDT_VALID,&st);

        if (bUseCustomTime)
          CheckRadioButton(hwnd,IDC_CURDATETIME,IDC_SPECDATETIME,IDC_SPECDATETIME);
        else {
          CheckRadioButton(hwnd,IDC_CURDATETIME,IDC_SPECDATETIME,IDC_CURDATETIME);
          EnableWindow(GetDlgItem(hwnd,IDC_DATE),FALSE);
          EnableWindow(GetDlgItem(hwnd,IDC_TIME),FALSE); }

        CheckRadioButton(hwnd,IDC_LASTACCESS,IDC_CREATION,(bModifyCreationTime) ? IDC_CREATION : IDC_LASTACCESS);

        ListView_SetImageList(GetDlgItem(hwnd,IDC_FILELIST),
          (HIMAGELIST)SHGetFileInfo(TEXT("C:\\"),0,&shfi,sizeof(SHFILEINFO),SHGFI_SMALLICON | SHGFI_SYSICONINDEX),
          LVSIL_SMALL);

        ListView_SetImageList(GetDlgItem(hwnd,IDC_FILELIST),
          (HIMAGELIST)SHGetFileInfo(TEXT("C:\\"),0,&shfi,sizeof(SHFILEINFO),SHGFI_LARGEICON | SHGFI_SYSICONINDEX),
          LVSIL_NORMAL);

        ListView_SetExtendedListViewStyle(GetDlgItem(hwnd,IDC_FILELIST),/*LVS_EX_DOUBLEBUFFER*/0x00010000|LVS_EX_LABELTIP);
        ListView_InsertColumn(GetDlgItem(hwnd,IDC_FILELIST),0,&lvc);

        AddFiles(GetDlgItem(hwnd,IDC_FILELIST),GetCommandLine(),TRUE);

        ListView_SetItemState(GetDlgItem(hwnd,IDC_FILELIST),0,LVIS_FOCUSED,LVIS_FOCUSED);
        ListView_SetColumnWidth(GetDlgItem(hwnd,IDC_FILELIST),0,LVSCW_AUTOSIZE_USEHEADER);

        EnableWindow(GetDlgItem(hwnd,IDOK),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //EnableWindow(GetDlgItem(hwnd,IDC_REMOVEFILES),ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //EnableWindow(GetDlgItem(hwnd,IDC_REMOVEALL),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
      }
      return TRUE;


    case WM_DESTROY:
    case WM_ENDSESSION:
      {
        RECT rc;
        RECT rcWorkArea;
        TCHAR tchIniFile[MAX_PATH];
        TCHAR tch[32];

        // Transform window rect from screen to workarea
        SystemParametersInfo(SPI_GETWORKAREA,0,&rcWorkArea,0);

        GetWindowRect(hwnd,&rc);
        xDialog = rc.left-rcWorkArea.left;
        yDialog = rc.top-rcWorkArea.top;
        cxDialog = rc.right-rc.left;
        cyDialog = rc.bottom-rc.top;

        // Save ini settings
        GetModuleFileName(GetModuleHandle(NULL),tchIniFile,MAX_PATH);
        PathRenameExtension(tchIniFile,TEXT(".ini"));

        if (PathFileExists(tchIniFile)) {

          wsprintf(tch,TEXT("%i"),xDialog);
          WritePrivateProfileString(TEXT("Settings"),TEXT("WinPosX"),tch,tchIniFile);
          wsprintf(tch,TEXT("%i"),yDialog);
          WritePrivateProfileString(TEXT("Settings"),TEXT("WinPosY"),tch,tchIniFile);
          wsprintf(tch,TEXT("%i"),cxDialog);
          WritePrivateProfileString(TEXT("Settings"),TEXT("WinSizeX"),tch,tchIniFile);
          wsprintf(tch,TEXT("%i"),cyDialog);
          WritePrivateProfileString(TEXT("Settings"),TEXT("WinSizeY"),tch,tchIniFile);

          wsprintf(tch,TEXT("%i"),bAlwaysOnTop);
          WritePrivateProfileString(TEXT("Settings"),TEXT("KeepOnTop"),tch,tchIniFile);
          wsprintf(tch,TEXT("%i"),bUseCustomTime);
          WritePrivateProfileString(TEXT("Settings"),TEXT("CustomTime"),tch,tchIniFile);
          wsprintf(tch,TEXT("%i"),bModifyCreationTime);
          WritePrivateProfileString(TEXT("Settings"),TEXT("ModifyCreationTime"),tch,tchIniFile);
        }
      }
      return FALSE;

    case WM_SIZE:
      {
        RECT rc;

        int dxClient = LOWORD(lParam) - cxClient;
        int dyClient = HIWORD(lParam) - cyClient;
        cxClient = LOWORD(lParam);
        cyClient = HIWORD(lParam);

        GetWindowRect(GetDlgItem(hwnd,IDC_GRIPPER),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDC_GRIPPER),NULL,rc.left+dxClient,rc.top+dyClient,0,0,SWP_NOZORDER|SWP_NOSIZE);
        InvalidateRect(GetDlgItem(hwnd,IDC_GRIPPER),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDOK),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDOK),NULL,rc.left+dxClient,rc.top+dyClient,0,0,SWP_NOZORDER|SWP_NOSIZE);
        InvalidateRect(GetDlgItem(hwnd,IDOK),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDCANCEL),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDCANCEL),NULL,rc.left+dxClient,rc.top+dyClient,0,0,SWP_NOZORDER|SWP_NOSIZE);
        InvalidateRect(GetDlgItem(hwnd,IDCANCEL),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDC_FRAME1),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDC_FRAME1),NULL,0,0,rc.right-rc.left+dxClient,rc.bottom-rc.top,SWP_NOZORDER|SWP_NOMOVE);
        InvalidateRect(GetDlgItem(hwnd,IDC_FRAME1),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDC_FRAME2),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDC_FRAME2),NULL,0,0,rc.right-rc.left+dxClient,rc.bottom-rc.top,SWP_NOZORDER|SWP_NOMOVE);
        InvalidateRect(GetDlgItem(hwnd,IDC_FRAME2),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDC_FRAME3),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDC_FRAME3),NULL,0,0,rc.right-rc.left+dxClient,rc.bottom-rc.top+dyClient,SWP_NOZORDER|SWP_NOMOVE);
        InvalidateRect(GetDlgItem(hwnd,IDC_FRAME3),NULL,TRUE);

        GetWindowRect(GetDlgItem(hwnd,IDC_FILELIST),&rc);
        MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        SetWindowPos(GetDlgItem(hwnd,IDC_FILELIST),NULL,0,0,rc.right-rc.left+dxClient,rc.bottom-rc.top+dyClient,SWP_NOZORDER|SWP_NOMOVE);
        ListView_SetColumnWidth(GetDlgItem(hwnd,IDC_FILELIST),0,LVSCW_AUTOSIZE_USEHEADER);
        InvalidateRect(GetDlgItem(hwnd,IDC_FILELIST),NULL,TRUE);

        //GetWindowRect(GetDlgItem(hwnd,IDC_ADDFILES),&rc);
        //MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        //SetWindowPos(GetDlgItem(hwnd,IDC_ADDFILES),NULL,rc.left+dxClient,rc.top,0,0,SWP_NOZORDER|SWP_NOSIZE);
        //InvalidateRect(GetDlgItem(hwnd,IDC_ADDFILES),NULL,TRUE);

        //GetWindowRect(GetDlgItem(hwnd,IDC_REMOVEFILES),&rc);
        //MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        //SetWindowPos(GetDlgItem(hwnd,IDC_REMOVEFILES),NULL,rc.left+dxClient,rc.top,0,0,SWP_NOZORDER|SWP_NOSIZE);
        //InvalidateRect(GetDlgItem(hwnd,IDC_REMOVEFILES),NULL,TRUE);

        //GetWindowRect(GetDlgItem(hwnd,IDC_REMOVEALL),&rc);
        //MapWindowPoints(NULL,hwnd,(LPPOINT)&rc,2);
        //SetWindowPos(GetDlgItem(hwnd,IDC_REMOVEALL),NULL,rc.left+dxClient,rc.top,0,0,SWP_NOZORDER|SWP_NOSIZE);
        //InvalidateRect(GetDlgItem(hwnd,IDC_REMOVEALL),NULL,TRUE);
      }
      return TRUE;


    case WM_GETMINMAXINFO:
      {
        LPMINMAXINFO lpmmi = (LPMINMAXINFO)lParam;
        lpmmi->ptMinTrackSize.x = mmiPtMinX;
        lpmmi->ptMinTrackSize.y = mmiPtMaxY;
        //lpmmi->ptMaxTrackSize.y = mmiPtMaxY;
      }
      return TRUE;


    case WM_DROPFILES:
      {
        int i;
        TCHAR tchFile[MAX_PATH+4];
        HDROP hDrop = (HDROP)wParam;
        int iFiles = DragQueryFile(hDrop,(UINT)-1,NULL,0);

        SetForegroundWindow(hwnd);

        for (i = 0; i < iFiles; i++)
        {
          DragQueryFile(hDrop,i,tchFile,MAX_PATH);

          if (PathIsDirectory(tchFile)) {

            if (!HIBYTE(GetKeyState(VK_SHIFT))) {

              PathQuoteSpaces(tchFile);
              AddFiles(GetDlgItem(hwnd,IDC_FILELIST),tchFile,FALSE);
              PathUnquoteSpaces(tchFile); }

            if (HIBYTE(GetKeyState(VK_SHIFT)) ||
                HIBYTE(GetKeyState(VK_CONTROL))) {

              PathAppend(tchFile,TEXT("*.*"));
              PathQuoteSpaces(tchFile);
              AddFiles(GetDlgItem(hwnd,IDC_FILELIST),tchFile,FALSE); }
          }

          else {

            PathQuoteSpaces(tchFile);
            AddFiles(GetDlgItem(hwnd,IDC_FILELIST),tchFile,FALSE);
          }
        }
        DragFinish(hDrop);
      }
      return TRUE;


    case WM_INITMENU:
      CheckMenuItem((HMENU)wParam,SC_ALWAYSONTOP,(bAlwaysOnTop) ? MF_CHECKED : MF_UNCHECKED);
      return TRUE;


    case WM_NOTIFY: {
      if (((LPNMHDR)(lParam))->idFrom == IDC_FILELIST) {

      switch (((LPNMHDR)(lParam))->code) {

        case NM_DBLCLK:
          SendMessage(hwnd,WM_COMMAND,MAKELONG(IDOK,1),0);
          break;


        case LVN_GETDISPINFO: {

          LV_DISPINFO *lpdi = (LPVOID)lParam;

          if (lpdi->item.mask & LVIF_IMAGE) {

            TCHAR tch[MAX_PATH];
            LV_ITEM lvi;
            SHFILEINFO shfi;
            DWORD dwFlags = SHGFI_SYSICONINDEX | SHGFI_ATTRIBUTES | SHGFI_ATTR_SPECIFIED;
            DWORD dwAttr  = 0;

            ZeroMemory(&lvi,sizeof(LV_ITEM));

            lvi.mask = LVIF_TEXT;
            lvi.pszText = tch;
            lvi.cchTextMax = MAX_PATH;
            lvi.iItem = lpdi->item.iItem;

            ListView_GetItem(GetDlgItem(hwnd,IDC_FILELIST),&lvi);

            if (!PathFileExists(tch)) {
              dwFlags |= SHGFI_USEFILEATTRIBUTES;
              dwAttr = FILE_ATTRIBUTE_NORMAL; }

            shfi.dwAttributes = SFGAO_LINK | SFGAO_SHARE;
            SHGetFileInfo(tch,dwAttr,&shfi,sizeof(SHFILEINFO),dwFlags);

            lpdi->item.iImage = shfi.iIcon;
            lpdi->item.mask |= LVIF_DI_SETITEM;

            lpdi->item.stateMask = 0;
            lpdi->item.state = 0;

            if (shfi.dwAttributes & SFGAO_LINK) {
              lpdi->item.mask |= LVIF_STATE;
              lpdi->item.stateMask |= LVIS_OVERLAYMASK;
              lpdi->item.state |= INDEXTOOVERLAYMASK(2); }

            if (shfi.dwAttributes & SFGAO_SHARE) {
              lpdi->item.mask |= LVIF_STATE;
              lpdi->item.stateMask |= LVIS_OVERLAYMASK;
              lpdi->item.state |= INDEXTOOVERLAYMASK(1); }

          dwAttr = GetFileAttributes(tch);

          if (dwAttr != INVALID_FILE_ATTRIBUTES &&
              dwAttr & (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM)) {
            lpdi->item.mask |= LVIF_STATE;
            lpdi->item.stateMask |= LVIS_CUT;
            lpdi->item.state |= LVIS_CUT; } } }

          break;


        case LVN_KEYDOWN: {

            NMLVKEYDOWN *lvkd = (LPVOID)lParam;

            if (lvkd->wVKey == VK_DELETE) {

              while (ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST))) {
                ListView_DeleteItem(GetDlgItem(hwnd,IDC_FILELIST),
                  ListView_GetNextItem(GetDlgItem(hwnd,IDC_FILELIST),-1,LVNI_SELECTED)); }

              EnableWindow(GetDlgItem(hwnd,IDOK),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
          } }
          break;


        case LVN_ITEMCHANGED:
        case LVN_INSERTITEM:
        case LVN_DELETEITEM:
          nIdFocus = GetDlgCtrlID(GetFocus());

          ListView_SetColumnWidth(GetDlgItem(hwnd,IDC_FILELIST),0,LVSCW_AUTOSIZE_USEHEADER);

          //if (nIdFocus == IDC_REMOVEFILES && !ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)))
          //  SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

          //if (nIdFocus == IDC_REMOVEALL && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
          //  SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

          //if (nIdFocus == IDOK && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
          //  SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

          EnableWindow(GetDlgItem(hwnd,IDOK),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
          //EnableWindow(GetDlgItem(hwnd,IDC_REMOVEFILES),ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)));
          //EnableWindow(GetDlgItem(hwnd,IDC_REMOVEALL),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
          break; } } }

      return TRUE;


    case WM_COMMAND:

      switch(LOWORD(wParam))
      {

        case IDOK:
          {
            SYSTEMTIME st;
            SYSTEMTIME st2;
            FILETIME   ftl;
            FILETIME   ft;

            int iItems, i;
            LVITEM      lvi;
            TCHAR       tchFile[MAX_PATH];

            BOOL bError = FALSE;

            if (IsDlgButtonChecked(hwnd,IDC_CURDATETIME))
            {
              GetLocalTime(&st);

              st.wSecond = 0;
              st.wMilliseconds = 0;
            }

            else
            {
              DateTime_GetSystemtime(GetDlgItem(hwnd,IDC_DATE),&st);
              DateTime_GetSystemtime(GetDlgItem(hwnd,IDC_TIME),&st2);

              st.wHour   = st2.wHour;
              st.wMinute = st2.wMinute;
              st.wSecond = st2.wSecond;
              st.wMilliseconds = 0;
            }

            SystemTimeToFileTime(&st,&ftl);
            LocalFileTimeToFileTime(&ftl,&ft);

            if (iItems = ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
            {
              ZeroMemory(&lvi,sizeof(LVITEM));

              for (i = 0; i < iItems; i++)
              {
                lvi.mask = LVIF_TEXT;
                lvi.iItem = i;
                lvi.pszText = tchFile;
                lvi.cchTextMax = MAX_PATH;

                ListView_GetItem(GetDlgItem(hwnd,IDC_FILELIST),&lvi);

                if (!SetFileTimeEx(tchFile,&ft,bModifyCreationTime))
                  bError = TRUE;
              }
              if (bError)
                MessageBox(hwnd,TEXT("Some of the files or directories couldn't be modified."),
                  TEXT("ntouch"),MB_ICONWARNING);
              else
                MessageBox(hwnd,TEXT("All of the files or directories properly modified."),
                  TEXT("ntouch"),MB_ICONINFORMATION);
            }
            else
              MessageBeep(0);
          }
          break;


        case IDCANCEL:
          EndDialog(hwnd,IDCANCEL);
          break;


        case IDC_CURDATETIME:
        case IDC_SPECDATETIME:
          if (IsDlgButtonChecked(hwnd,IDC_CURDATETIME)) {
            if (IsWindowEnabled(GetDlgItem(hwnd,IDC_DATE))) {
              EnableWindow(GetDlgItem(hwnd,IDC_DATE),FALSE);
              EnableWindow(GetDlgItem(hwnd,IDC_TIME),FALSE);
              bUseCustomTime = FALSE; } }
          else {
            if (!IsWindowEnabled(GetDlgItem(hwnd,IDC_DATE))) {
              EnableWindow(GetDlgItem(hwnd,IDC_DATE),TRUE);
              EnableWindow(GetDlgItem(hwnd,IDC_TIME),TRUE);
              bUseCustomTime = TRUE; } }
          break;


        case IDC_CREATION:
        case IDC_LASTACCESS:
          if (IsDlgButtonChecked(hwnd,IDC_CREATION))
            bModifyCreationTime = TRUE;
          else
            bModifyCreationTime = FALSE;
          break;


        //case IDC_ADDFILES:
        //  {
        //    TCHAR tchFile[MAX_PATH*100];
        //    TCHAR tchItem[MAX_PATH];
        //    TCHAR *p;
        //    OPENFILENAME ofn;

        //    ZeroMemory(&ofn,sizeof(OPENFILENAME));
        //    StrCpy(tchFile,TEXT(""));

        //    ofn.lStructSize = sizeof(OPENFILENAME);
        //    ofn.hwndOwner = hwnd;
        //    ofn.lpstrFilter = TEXT("All Files (*.*)\0*.*\0\0");
        //    ofn.lpstrFile = tchFile;
        //    ofn.nMaxFile = MAX_PATH*100;
        //    ofn.lpstrTitle = TEXT("Add Files");
        //    ofn.Flags = OFN_ALLOWMULTISELECT | OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
        //                OFN_PATHMUSTEXIST | OFN_SHAREAWARE | OFN_NODEREFERENCELINKS | /*OFN_DONTADDTORECENT*/0x02000000;

        //    if (GetOpenFileName(&ofn))
        //    {

        //      if (PathIsDirectory(tchFile))
        //      {
        //        p = tchFile;
        //        while (*p++) // now point to last 0

        //        while (*(p+1)) {
        //          StrCpyN(tchItem,tchFile,MAX_PATH);
        //          PathAppend(tchItem,++p);
        //          PathQuoteSpaces(tchItem);
        //          AddFiles(GetDlgItem(hwnd,IDC_FILELIST),tchItem,FALSE);
        //          while (*p) p++; }
        //      }
        //      else
        //      {
        //        PathQuoteSpaces(tchFile);
        //        AddFiles(GetDlgItem(hwnd,IDC_FILELIST),tchFile,FALSE);
        //      }
        //    }
        //  }
        //  break;


        //case IDC_REMOVEFILES:
        //  while (ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST))) {
        //    ListView_DeleteItem(GetDlgItem(hwnd,IDC_FILELIST),
        //      ListView_GetNextItem(GetDlgItem(hwnd,IDC_FILELIST),-1,LVNI_SELECTED)); }

        //  nIdFocus = GetDlgCtrlID(GetFocus());

        //  if (nIdFocus == IDC_REMOVEFILES && !ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  if (nIdFocus == IDC_REMOVEALL && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  if (nIdFocus == IDOK && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  EnableWindow(GetDlgItem(hwnd,IDOK),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  EnableWindow(GetDlgItem(hwnd,IDC_REMOVEFILES),ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  EnableWindow(GetDlgItem(hwnd,IDC_REMOVEALL),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  break;

        //case IDC_REMOVEALL:
        //  ListView_DeleteAllItems(GetDlgItem(hwnd,IDC_FILELIST));

        //  nIdFocus = GetDlgCtrlID(GetFocus());

        //  if (nIdFocus == IDC_REMOVEFILES && !ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  if (nIdFocus == IDC_REMOVEALL && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  if (nIdFocus == IDOK && !ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)))
        //    SendMessage(hwnd,WM_NEXTDLGCTL,(WPARAM)(GetDlgItem(hwnd,IDC_ADDFILES)),TRUE);

        //  EnableWindow(GetDlgItem(hwnd,IDOK),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  EnableWindow(GetDlgItem(hwnd,IDC_REMOVEFILES),ListView_GetSelectedCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  EnableWindow(GetDlgItem(hwnd,IDC_REMOVEALL),ListView_GetItemCount(GetDlgItem(hwnd,IDC_FILELIST)));
        //  break;

      }

      return TRUE;


    case WM_SYSCOMMAND:

      switch (wParam) {

        case SC_ALWAYSONTOP:
          if (bAlwaysOnTop) {
            SetWindowPos(hwnd,HWND_NOTOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
            bAlwaysOnTop = FALSE; }
          else {
            SetWindowPos(hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
            bAlwaysOnTop = TRUE; }
          return TRUE;

        //case SC_INFO: {

        //  MSGBOXPARAMS mbp;

        //  mbp.cbSize = sizeof(MSGBOXPARAMS);
        //  mbp.hwndOwner = hwnd;
        //  mbp.hInstance = GetModuleHandle(NULL);
        //  mbp.lpszText = TEXT("ntouch 2.0\n\n(c) Florian Balmer 2001-2003\nhttp://www.flos-freeware.ch");
        //  mbp.lpszCaption = TEXT("ntouch");
        //  mbp.dwStyle = MB_OK | MB_USERICON | MB_SETFOREGROUND;
        //  mbp.lpszIcon = MAKEINTRESOURCE(IDI_NTOUCH);
        //  mbp.dwContextHelpId = 0;
        //  mbp.lpfnMsgBoxCallback = NULL;
        //  mbp.dwLanguageId = MAKELANGID(LANG_NEUTRAL,SUBLANG_NEUTRAL);

        //  MessageBoxIndirect(&mbp); }

        //  return TRUE;

        default:
          return FALSE;
      }
      return FALSE;


  }

  return FALSE;

}


#ifndef DLGTEMPLATEEX
#pragma pack(push, 1)
typedef struct {
  WORD      dlgVer;
  WORD      signature;
  DWORD     helpID;
  DWORD     exStyle;
  DWORD     style;
  WORD      cDlgItems;
  short     x;
  short     y;
  short     cx;
  short     cy;
} DLGTEMPLATEEX;
#pragma pack(pop)
#endif

BOOL GetThemedDialogFont(LPWSTR lpFaceName,WORD* wSize)
{
  HDC hDC;
  int iLogPixelsY;
  HMODULE hModUxTheme;
  HTHEME hTheme;
  LOGFONT lf;
  BOOL bSucceed = FALSE;

  hDC = GetDC(NULL);
  iLogPixelsY = GetDeviceCaps(hDC,LOGPIXELSY);
  ReleaseDC(NULL,hDC);

  if (hModUxTheme = GetModuleHandle(L"uxtheme.dll")) {
    if ((BOOL)(GetProcAddress(hModUxTheme,"IsAppThemed"))()) {
      hTheme = (HTHEME)(GetProcAddress(hModUxTheme,"OpenThemeData"))(NULL,L"WINDOWSTYLE;WINDOW");
      if (hTheme) {
        if (S_OK == (HRESULT)(GetProcAddress(hModUxTheme,"GetThemeSysFont"))(hTheme,/*TMT_MSGBOXFONT*/805,&lf)) {
          if (lf.lfHeight < 0)
            lf.lfHeight = -lf.lfHeight;
          *wSize = (WORD)MulDiv(lf.lfHeight,72,iLogPixelsY);
          if (*wSize == 0)
            *wSize = 8;
          StrCpyN(lpFaceName,lf.lfFaceName,LF_FACESIZE);
          bSucceed = TRUE;
        }
        (GetProcAddress(hModUxTheme,"CloseThemeData"))(hTheme);
      }
    }
  }

  /*
  if (!bSucceed) {
    NONCLIENTMETRICS ncm;
    ncm.cbSize = sizeof(NONCLIENTMETRICS);
    SystemParametersInfo(SPI_GETNONCLIENTMETRICS,sizeof(NONCLIENTMETRICS),&ncm,0);
    if (ncm.lfMessageFont.lfHeight < 0)
      ncm.lfMessageFont.lfHeight = -ncm.lfMessageFont.lfHeight;
    *wSize = (WORD)MulDiv(ncm.lfMessageFont.lfHeight,72,iLogPixelsY);
    if (*wSize == 0)
      *wSize = 8;
    StrCpyN(lpFaceName,ncm.lfMessageFont.lfFaceName,LF_FACESIZE);
  }*/

  return(bSucceed);
}

__inline BOOL DialogTemplate_IsDialogEx(const DLGTEMPLATE* pTemplate) {

  return ((DLGTEMPLATEEX*)pTemplate)->signature == 0xFFFF;
}

__inline BOOL DialogTemplate_HasFont(const DLGTEMPLATE* pTemplate) {

  return (DS_SETFONT &
    (DialogTemplate_IsDialogEx(pTemplate) ? ((DLGTEMPLATEEX*)pTemplate)->style : pTemplate->style));
}

__inline int DialogTemplate_FontAttrSize(BOOL bDialogEx) {

  return (int)sizeof(WORD) * (bDialogEx ? 3 : 1);
}

__inline BYTE* DialogTemplate_GetFontSizeField(const DLGTEMPLATE* pTemplate) {

  BOOL bDialogEx = DialogTemplate_IsDialogEx(pTemplate);
  WORD* pw;

  if (bDialogEx)
    pw = (WORD*)((DLGTEMPLATEEX*)pTemplate + 1);
  else
    pw = (WORD*)(pTemplate + 1);

  if (*pw == (WORD)-1)
    pw += 2;
  else
    while(*pw++);

  if (*pw == (WORD)-1)
    pw += 2;
  else
    while(*pw++);

  while (*pw++);

  return (BYTE*)pw;
}

DLGTEMPLATE* LoadThemedDialogTemplate(LPCTSTR lpDialogTemplateID,HINSTANCE hInstance) {

  HRSRC hRsrc;
  HGLOBAL hRsrcMem;
  DLGTEMPLATE *pRsrcMem;
  DLGTEMPLATE *pTemplate;
  UINT dwTemplateSize = 0;
  WCHAR wchFaceName[LF_FACESIZE];
  WORD wFontSize;
  BOOL bDialogEx;
  BOOL bHasFont;
  int cbFontAttr;
  int cbNew;
  int cbOld;
  BYTE* pbNew;
  BYTE* pb;
  BYTE* pOldControls;
  BYTE* pNewControls;
  WORD nCtrl;

  hRsrc = FindResource(hInstance,lpDialogTemplateID,RT_DIALOG);
  if (hRsrc == NULL)
    return(NULL);

  hRsrcMem = LoadResource(hInstance,hRsrc);
  pRsrcMem = (DLGTEMPLATE*)LockResource(hRsrcMem);
  dwTemplateSize = (UINT)SizeofResource(hInstance,hRsrc);

  if ((dwTemplateSize == 0) ||
      (pTemplate = LocalAlloc(LPTR,dwTemplateSize+LF_FACESIZE*2)) == NULL) {
    UnlockResource(hRsrcMem);
    FreeResource(hRsrcMem);
    return(NULL);
  }

  CopyMemory((BYTE*)pTemplate,pRsrcMem,(size_t)dwTemplateSize);
  UnlockResource(hRsrcMem);
  FreeResource(hRsrcMem);

  if (!GetThemedDialogFont(wchFaceName,&wFontSize))
    return(pTemplate);

  bDialogEx  = DialogTemplate_IsDialogEx(pTemplate);
  bHasFont   = DialogTemplate_HasFont(pTemplate);
  cbFontAttr = DialogTemplate_FontAttrSize(bDialogEx);

  if (bDialogEx)
    ((DLGTEMPLATEEX*)pTemplate)->style |= DS_SHELLFONT;
  else
    pTemplate->style |= DS_SHELLFONT;

  cbNew = cbFontAttr + ((lstrlen(wchFaceName) + 1) * sizeof(WCHAR));
  pbNew = (BYTE*)wchFaceName;

  pb = DialogTemplate_GetFontSizeField(pTemplate);
  cbOld = (int)(bHasFont ? cbFontAttr + 2 * (lstrlen((WCHAR*)(pb + cbFontAttr)) + 1) : 0);

  pOldControls = (BYTE*)(((DWORD_PTR)pb + cbOld + 3) & ~(DWORD_PTR)3);
  pNewControls = (BYTE*)(((DWORD_PTR)pb + cbNew + 3) & ~(DWORD_PTR)3);

  nCtrl = bDialogEx ?
    (WORD)((DLGTEMPLATEEX*)pTemplate)->cDlgItems :
    (WORD)pTemplate->cdit;

  if (cbNew != cbOld && nCtrl > 0)
    MoveMemory(pNewControls,pOldControls,(size_t)(dwTemplateSize - (pOldControls - (BYTE*)pTemplate)));

  *(WORD*)pb = wFontSize;
  MoveMemory(pb + cbFontAttr,pbNew,(size_t)(cbNew - cbFontAttr));

  return(pTemplate);
}

INT_PTR ThemedDialogBoxParam(
  HINSTANCE hInstance,
  LPCTSTR lpTemplate,
  HWND hWndParent,
  DLGPROC lpDialogFunc,
  LPARAM dwInitParam) {

  INT ret;
  DLGTEMPLATE *pDlgTemplate;

  pDlgTemplate = LoadThemedDialogTemplate(lpTemplate,hInstance);
  ret = DialogBoxIndirectParam(hInstance,pDlgTemplate,hWndParent,lpDialogFunc,dwInitParam);
  if (pDlgTemplate)
    LocalFree(pDlgTemplate);

  return(ret);
}


#pragma comment(linker,"/entry:ntouch")
void ntouch()
{

  WNDCLASS wc;

  INITCOMMONCONTROLSEX icc = { sizeof(INITCOMMONCONTROLSEX),
                               ICC_DATE_CLASSES };

  HINSTANCE hInstance = GetModuleHandle(NULL);

  InitCommonControlsEx(&icc);

  CoInitialize(NULL);

  GetClassInfo(NULL,MAKEINTRESOURCE(32770),&wc);
  wc.hIcon = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_NTOUCH));
  wc.lpszClassName = TEXT("ntouch");
  RegisterClass(&wc);

  ThemedDialogBoxParam(
    hInstance,
    MAKEINTRESOURCE(IDD_NTOUCH),
    NULL,
    MainDlgProc,
    0);
  UnregisterClass(TEXT("ntouch"),hInstance);

  CoUninitialize();

  ExitProcess(0);

}
